/*
 * Copyright 2011 NEHTA
 *
 * Licensed under the NEHTA Open Source (Apache) License; you may not use this
 * file except in compliance with the License. A copy of the License is in the
 * 'license.txt' file, which should be provided with this work.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package au.gov.nehta.vendorlibrary.hi.ihi;


import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import javax.net.ssl.SSLSocketFactory;
import javax.xml.ws.Holder;

import au.gov.nehta.vendorlibrary.hi.client.BaseClient_30;
import au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.ProductType;
import au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.SignatureContainerType;
import au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.TimestampType;
import au.net.electronichealth.ns.hi.common.qualifiedidentifier._3_0.QualifiedId;
import au.net.electronichealth.ns.hi.svc.consumersearchihibatchsync._3_0.ConsumerSearchIHIBatchSyncPortType;
import au.net.electronichealth.ns.hi.svc.consumersearchihibatchsync._3_0.ConsumerSearchIHIBatchSyncService;
import au.net.electronichealth.ns.hi.svc.consumersearchihibatchsync._3_0.SearchIHIBatchResponse;
import au.net.electronichealth.ns.hi.svc.consumersearchihibatchsync._3_0.StandardErrorMsg;
/**
 * An implementation of a Healthcare Identifiers (HI) - Individual Healthcare Identifier (IHI) batch search client.
 * This class may be used to connect to the Medicare HI Service to do Consumer Search IHI Synchronized Batch
 * searches.
 */
public class ConsumerSearchIHIBatchSyncClient extends BaseClient_30<ConsumerSearchIHIBatchSyncPortType> {
 
            
      /**
       * Constructor which creates a new ConsumerSearchIHIBatchSyncClient with an endpoint and an SSL Socket Factory.
       *
       * @param searchIhiServiceEndpoint the Web Service endpoint for the Medicare HI Service interface (Mandatory)
       * @param individualQualifiedId    The qualified user ID for connecting to the ConsumerSearchIHIBatchSync service (Mandatory)
       * @param productHeader            The product header data for connecting to the ConsumerSearchIHIBatchSync service (Mandatory)
       * @param signingPrivateKey        The private key to be used for signing (Mandatory)
       * @param signingCertificate       The certificate to be used for signing (Mandatory)
       * @param sslSocketFactory         the SSL Socket Factory to be used when connecting to the Web Service provider (Mandatory)
       */
      public ConsumerSearchIHIBatchSyncClient(final String searchIhiServiceEndpoint,
                                              final QualifiedId individualQualifiedId,
                                              final Holder<ProductType> productHeader,
                                              final PrivateKey signingPrivateKey,
                                              final X509Certificate signingCertificate,
                                              final SSLSocketFactory sslSocketFactory) {
        this(
          searchIhiServiceEndpoint,
          individualQualifiedId,
          null,
          productHeader,
          signingPrivateKey,
          signingCertificate,
          sslSocketFactory
        );
      }
    
      /**
       * Constructor which creates a new ConsumerSearchIHIBatchSyncClient with an endpoint and an SSL Socket Factory, with
       * the optional contracted service providers HPI-O organisation qualified ID set.
       *
       * @param serviceEndpoint the Web Service endpoint for the Medicare HI Service interface (Mandatory)
       * @param individualQualifiedId    The qualified user ID for connecting to the ConsumerSearchIHIBatchSync service (Mandatory)
       * @param organisationQualifiedId  The qualified organisation ID for connecting to the ConsumerSearchIHIBatchSync service (Optional)
       * @param productHeader            The product header data for connecting to the ConsumerSearchIHIBatchSync service (Mandatory)
       * @param signingPrivateKey        The private key to be used for signing (Mandatory)
       * @param signingCertificate       The certificate to be used for signing (Mandatory)
       * @param sslSocketFactory         the SSL Socket Factory to be used when connecting to the Web Service provider (Mandatory)
       */
      private ConsumerSearchIHIBatchSyncClient(final String serviceEndpoint,
                                              final QualifiedId individualQualifiedId,
                                              final QualifiedId organisationQualifiedId,
                                              final Holder<ProductType> productHeader,
                                              final PrivateKey signingPrivateKey,
                                              final X509Certificate signingCertificate,
                                              final SSLSocketFactory sslSocketFactory) {
    
          super(  ConsumerSearchIHIBatchSyncPortType.class,
                  ConsumerSearchIHIBatchSyncService.class,
                  individualQualifiedId,
                  organisationQualifiedId,
                  productHeader,
                  signingPrivateKey,
                  signingCertificate,
                  sslSocketFactory,
                  serviceEndpoint
                  );
      }
    
      /**
       * Executes a SearchIHIBatchSync search.
       *
       * @param request the SearchBatch request object containing the searches to be performed.
       * @return the response from the SearchIHIBatchSync service
       * @throws StandardErrorMsg if the Web Service call fails.
       */
      public final SearchIHIBatchResponse batchSearch(SearchBatch request) throws StandardErrorMsg {
        TimestampType timestampHeader = getTimestampHeader();
        Holder<SignatureContainerType> signatureHeader = null;
        return getPort().searchIHIBatchSync(
          request.getBatch(),
          this.productHeader,
          timestampHeader,
          signatureHeader,
          this.individualQualifiedId,
          this.organisationQualifiedId
        );
      }
}
