package au.gov.nehta.vendorlibrary.hi.client;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.SSLSocketFactory;
import javax.xml.ws.Holder;
import javax.xml.ws.Service;
import javax.xml.ws.handler.Handler;

import au.gov.nehta.common.utils.ArgumentUtils;
import au.gov.nehta.vendorlibrary.ws.TimeUtility;
import au.gov.nehta.vendorlibrary.ws.WebServiceClientUtil;
import au.gov.nehta.vendorlibrary.ws.handler.LoggingHandler;
import au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.ProductType;
import au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.TimestampType;
import au.net.electronichealth.ns.hi.common.qualifiedidentifier._3_0.QualifiedId;


/**
 * A Base implementation of the client wrapper that allows access to the 
 * WS Port for the XXXXX._30 series of classes
 * 
 * @author NeHTA
 */
public class BaseClient_30<T> extends ClientBase<T>{
   
    
    /**
     * The user Qualified ID associated with this use of the ProviderSearchHIProviderDirectoryForOrganisation service
     */
    protected final QualifiedId individualQualifiedId;
  
    /**
     * The organisation Qualified ID associated with this use of the ProviderSearchHIProviderDirectoryForOrganisation service
     */
    protected final QualifiedId organisationQualifiedId;
    
    
    /**
     * The Product details associated with this use of the ProviderSearchHIProviderDirectoryForIndividual service
     */
    protected final Holder<ProductType> productHeader;
  
    
    
    public BaseClient_30(
            final Class<T> portClass,
            final Class<? extends Service> serviceClass,
            final QualifiedId individualQualifiedId2,
            final QualifiedId organisationQualifiedId2, 
            final Holder<ProductType> productHeader2,
            final PrivateKey signingPrivateKey,
            final X509Certificate signingCertificate, 
            final SSLSocketFactory sslSocketFactory,
            final String serviceEndpoint
            ) {
        
        ArgumentUtils.checkNotNull(serviceClass, "serviceClass");
        ArgumentUtils.checkNotNull(portClass, "portClass");
        ArgumentUtils.checkNotNullNorBlank(serviceEndpoint, "serviceEndpoint");
        ArgumentUtils.checkNotNull(individualQualifiedId2, "individualQualifiedId");
        ArgumentUtils.checkNotNull(productHeader2, "productHeader");
        ArgumentUtils.checkNotNull(signingPrivateKey, "signingPrivateKey");
        ArgumentUtils.checkNotNull(signingCertificate, "signingPrivateKey");
        ArgumentUtils.checkNotNull(sslSocketFactory, "sslSocketFactory");
        
        this.productHeader = productHeader2;
        this.organisationQualifiedId=organisationQualifiedId2;
        this.individualQualifiedId=individualQualifiedId2;
        this.signingPrivateKey = signingPrivateKey;
        this.signingCertificate = signingCertificate;
        this.sslSocketFactory = sslSocketFactory;
        this.loggingHandler = new LoggingHandler(true); //Set true to dump the SOAP message to the default logger.
        
        
        @SuppressWarnings("rawtypes")
        List<Handler> handlerChain = new ArrayList<Handler>();
        handlerChain.add(loggingHandler);
        
        this.port =(T) WebServiceClientUtil.getPort(
                portClass,
                serviceClass,
                sslSocketFactory,
                handlerChain);
        
        configureEndpoint(this.port, serviceEndpoint);
    }

    /**
     * Returns the current
     * {@link au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.TimestampType}
     * 
     * @return {@link au.net.electronichealth.ns.hi.common.commoncoreelements._3_0.TimestampType}
     *         instance with current as created time.
     */
    protected TimestampType getTimestampHeader() {
        TimestampType timestampHeader = new TimestampType();
        timestampHeader.setCreated(TimeUtility.nowXMLGregorianCalendar());
        return timestampHeader;
    }

    @Override
    protected boolean hasNoOrganisationQualifiedId() {
        return organisationQualifiedId == null;
    }
}
